require 'import'
require 'csp-am'
require 'tenant/clients/csp_base_client'
require 'tenant/errors/tenant_service_error'
require 'cht-lib-fabric/helpers/wavefront'

module Tenant
  module Clients
    class CspOrgClient < CspBaseClient
      include Import['logger', 'settings', :wavefront], ::Tenant::Clients
      include ::Platform::Helpers::Wavefront

      CSP_ORG_DISPLAY_NAME_SUFFIX = "CloudHealth"
      CSP_ORG_OWNER_ROLE = "org_owner"
      CH_SERVICE_DEFAULT_GRANT_ROLE = "ch_platform:administrator"
      CH_SERVICE_DEFINITION_LINK = "/csp/gateway/slc/api/definitions/external/"
      ORG_DISPLAY_NAME_NOT_VALID_MESSAGE = "Validation failed for request body: displayName must be alphanumerical and can contain the following symbols"

      # This method is used to create a CSP organization for a given tenant.
      # Optional display name will be provided with pattern: tenant_id-tenant_name-CloudHealth
      # Authentication will be done using a csp.local domain account.
      def create_csp_organization(tenant_id, tenant_name)
        tenant_org_display_name = tenant_id.to_s + "-" + tenant_name + "-" + CSP_ORG_DISPLAY_NAME_SUFFIX

        begin
          return create_csp_organization_with_display_name(tenant_id, tenant_org_display_name)
        rescue => ex
          logger.error "Unable to create csp organization for customer #{tenant_id}. Error: #{ex.message}"
          if !ex.response_body.nil? and ex.response_body.include? ORG_DISPLAY_NAME_NOT_VALID_MESSAGE
            tenant_org_display_name = tenant_id.to_s + "-" + CSP_ORG_DISPLAY_NAME_SUFFIX
            logger.info "Original display name not valid for tenant ID #{tenant_id}. Going to create csp organization
 without CH customer name in display name. New display name: #{tenant_org_display_name}"
            return create_csp_organization_with_display_name(tenant_id, tenant_org_display_name)
          end

          raise ex
        end
      end

      # This method is used to invite CH tenant admins to their csp organization.
      # By default org_owner role will be provided.
      # Additionally, they would also be granted service admin role for CloudHealth service in csp.
      def invite_users_to_csp_organization(csp_organization_id, users)
        org_api = get_org_api(csp_organization_id)
        logger.info "Going to send user invitations for csp org #{csp_organization_id} to users #{users}"
        user_invitation_request = create_user_invitation_request(users)

        wf_time(WAVEFRONT_CSP_API_TIME_TAKEN_METRIC, tags: {:name => "invite_users"}) do
          org_api.invite_users_using_post_with_http_info(csp_organization_id, user_invitation_request)
        end

        wavefront.counter(WAVEFRONT_CSP_API_HIT_COUNT_METRIC, 1, name: 'invite_users')

        logger.info "Invited users #{users} for csp organization #{csp_organization_id}"
      end

      # This method is used to remove csp.local system account from tenant csp organization.
      # This will be invoked when a tenant admin becomes org_owner in tenant csp organization.
      def remove_users_from_csp_organization(tenant_id, csp_organization_id, users)
        org_api = get_org_api(csp_organization_id)
        logger.info "Going to remove users #{users} from csp organization #{csp_organization_id}"
        remove_users_from_org_request = create_remove_users_from_organization_request(users)

        response = nil
        wf_time(WAVEFRONT_CSP_API_TIME_TAKEN_METRIC, tags: {:name => "remove_users"}) do
          response = org_api.remove_users_from_organization_using_delete_with_http_info(csp_organization_id, remove_users_from_org_request)
        end

        wavefront.counter(WAVEFRONT_CSP_API_HIT_COUNT_METRIC, 1, name: 'remove_users')

        logger.info "Removed users #{users} from csp organization #{csp_organization_id}"

        if response[0].failed.length() > 0
          raise ::Tenant::Errors::TenantServiceError.new(tenant_id, "Failed to remove system account from CSP organization", csp_organization_id)
        end
      end

      # This method is used to fetch users list for a csp organization.
      # This will be used to check if a tenant admin has become org_owner in tenant csp organization.
      def get_users_in_csp_organization(csp_organization_id)
        begin
          org_api_v2 = get_org_api_v2(csp_organization_id)
          logger.info "Going to fetch users for csp organization #{csp_organization_id}"

          response = nil
          wf_time(WAVEFRONT_CSP_API_TIME_TAKEN_METRIC, tags: {:name => "get_org_users"}) do
            response = org_api_v2.get_paginated_org_users_info_using_get_with_http_info(csp_organization_id)
          end

          wavefront.counter(WAVEFRONT_CSP_API_HIT_COUNT_METRIC, 1, name: 'get_org_users')

          logger.info "Fetched users for csp organization #{csp_organization_id}"
          return response[0]
        rescue => error
          logger.error "Unable to fetch users for csp organization #{csp_organization_id}. Response code: #{error.code}, Response message: #{error.response_body}"
          return nil
        end
      end

      # This method is used to fetch csp organization details.
      # This will be used to check if a csp organization already exists for a tenant.
      def get_csp_organization_details(csp_organization_id)
        org_api = get_org_api(csp_organization_id)
        logger.info "Going to fetch details for csp organization #{csp_organization_id}"

        response = nil
        wf_time(WAVEFRONT_CSP_API_TIME_TAKEN_METRIC, tags: {:name => "get_org"}) do
          response = org_api.get_org_using_get_with_http_info(csp_organization_id)
        end

        wavefront.counter(WAVEFRONT_CSP_API_HIT_COUNT_METRIC, 1, name: 'get_org')

        logger.info "Fetched details for csp organization #{csp_organization_id}"
        return response[0]
      end

      def find_number_of_org_owners(users_dto, tenant_id, csp_organization_id)
        if users_dto.nil? or users_dto.results.nil?
          raise ::Tenant::Errors::TenantServiceError.new(tenant_id, "There are no users in csp organization", csp_organization_id)
        end

        count = 0;
        users_dto.results.each do |user_dto|
          if is_org_owner_role_present(user_dto.organization_roles)
            count = count+1
          end
        end

        return count
      end

      def extract_tenant_id_from_org_display_name(csp_organization)
        if csp_organization.display_name.nil?
          return nil
        end

        display_name = csp_organization.display_name
        display_name_array = display_name.split('-')
        if display_name_array.length() != 3
          return nil
        end

        return display_name_array[0]
      end

      private

      def create_csp_organization_with_display_name(tenant_id, display_name)
        default_org = settings.csp_default_org

        org_api = get_org_api(default_org)
        logger.info "Going to create csp organization for tenant ID #{tenant_id}"
        create_org_request = ::CspAm::CreateOrganizationRequest.new({ display_name: display_name, usernames: ['cloudhealth@csp.local'] })

        response = nil
        wf_time(WAVEFRONT_CSP_API_TIME_TAKEN_METRIC, tags: {:name => "create_org"}) do
          response = org_api.create_org_using_post_with_http_info(create_org_request)
        end

        wavefront.counter(WAVEFRONT_CSP_API_HIT_COUNT_METRIC, 1, name: 'create_org')

        logger.info "Created csp organization for tenant ID #{tenant_id}"
        if !response.nil? and !response[0].nil? and !response[0].ref_link.nil?
          return response[0].ref_link.split('/').last()
        end

        return nil
      end

      def get_org_api(csp_organization_id)
        am = get_csp_am_client(csp_organization_id)
        return ::CspAm::OrganizationApi::new am
      end

      def get_org_api_v2(csp_organization_id)
        am = get_csp_am_client(csp_organization_id)
        return ::CspAm::OrganizationV2Api::new am
      end

      def get_csp_am_client(csp_organization_id)
        access_token = self.auth_client.fetch_access_token_using_oauth(csp_organization_id)
        return self.api_client.am(access_token)
      end

      def create_organization_metadata_update_request(tenant_id, tenant_name)
        tenant_org_display_name = tenant_id.to_s + "-" + tenant_name + "-" + CSP_ORG_DISPLAY_NAME_SUFFIX
        metadata = {}
        metadata[:'ch_tenant_id'] = tenant_id
        attributes = {}
        attributes[:'metadata'] = metadata
        attributes[:'display_name'] = tenant_org_display_name
        organization_dto = ::CspAm::OrganizationDto::new(attributes)
        return organization_dto
      end

      def create_user_invitation_request(users)
        org_role = ::CspAm::BaseRoleBindingDto::new({:'name' => CSP_ORG_OWNER_ROLE})
        attributes = {}
        attributes[:'organization_roles'] = [org_role]
        attributes[:'usernames'] = users
        attributes[:'invited_by'] = settings.csp_account_username
        attributes[:'skip_notify'] = true
        attributes[:'skip_notify_registration'] = true

        service_role_binding = ::CspAm::ServiceRoleBindingDto::new({:'name' => CH_SERVICE_DEFAULT_GRANT_ROLE})
        service_role_bindings = [service_role_binding]

        service_role = ::CspAm::ServiceRolesDto::new({:'service_definition_link' => CH_SERVICE_DEFINITION_LINK + settings.ch_service_definition_id, :'service_roles' => service_role_bindings})
        service_roles = [service_role]

        attributes[:'service_roles_dtos'] = service_roles

        user_invitation_request = ::CspAm::UsersOrganizationInvitationRequest::new(attributes)
        return user_invitation_request
      end

      def create_remove_users_from_organization_request(users)
        attributes = {:'users' => users}
        remove_users_from_org_request = ::CspAm::RemoveUsersFromOrganizationRequest::new(attributes)
        return remove_users_from_org_request
      end

      def is_org_owner_role_present(organization_roles)
        if organization_roles.nil? or organization_roles.length() == 0
          return false
        end

        organization_roles.each do |role|
          if CSP_ORG_OWNER_ROLE === role.name
            return true
          end
        end

        return false
      end

    end
  end
end