require 'import'
require 'csp-am'
require 'csp-slc'
require 'tenant/clients/csp_base_client'
require 'cht-lib-fabric/helpers/wavefront'

module Tenant
  module Clients
    class CspSlcClient < CspBaseClient
      include Import['logger', 'settings', :wavefront], ::Tenant::Clients
      include ::Platform::Helpers::Wavefront

      SERVICE_ACCESS_ALREADY_GRANTED_MESSAGE = "The org already has access to this service."
      SERVICE_METADATA_ALREADY_EXISTS_MESSAGE = "Service Metadata already exists"

      # This method is used to grant access to CloudHealth service in csp to tenant csp organization.
      def grant_service_access(csp_organization_id, service_definition_id)
        begin
          slc = get_slc_api(settings.csp_default_org)
          slc_api = ::CspSlc::ServiceAccessApi::new slc

          logger.info "Granting CloudHealth service access to csp org #{csp_organization_id}"
          service_access_request = create_grant_service_access_request(csp_organization_id, service_definition_id)

          wf_time(WAVEFRONT_CSP_API_TIME_TAKEN_METRIC, tags: {:name => "grant_service_access"}) do
            slc_api.grant_service_access_using_post_with_http_info(service_access_request)
          end

          wavefront.counter(WAVEFRONT_CSP_API_HIT_COUNT_METRIC, 1, name: 'grant_service_access')

          logger.info "Granted CloudHealth service access to csp org #{csp_organization_id}"
        rescue => ex
          if ex.response_body.nil? or !ex.response_body.include? SERVICE_ACCESS_ALREADY_GRANTED_MESSAGE
            raise ex
          end

          logger.info "CloudHealth service access is already granted to csp org #{csp_organization_id}"
        end
      end

      # This method is used to store CH tenant id as metadata in tenant organization.
      def add_service_organization_metadata(tenant_id, csp_organization_id, service_definition_id)
        begin
          slc = get_slc_api(settings.csp_default_org)
          slc_api = ::CspSlc::ServiceMetadataApi::new slc

          logger.info "Going to update service metadata for tenant id #{tenant_id}"
          metadata = {}
          metadata['CH_TENANT_ID'] = tenant_id

          wf_time(WAVEFRONT_CSP_API_TIME_TAKEN_METRIC, tags: {:name => "create_service_metadata"}) do
            slc_api.create_service_metadata_using_post_with_http_info(service_definition_id, csp_organization_id, metadata)
          end

          wavefront.counter(WAVEFRONT_CSP_API_HIT_COUNT_METRIC, 1, name: 'create_service_metadata')

          logger.info "Updated service metadata for tenant id #{tenant_id}"
        rescue => ex
          if ex.response_body.nil? or !ex.response_body.include? SERVICE_METADATA_ALREADY_EXISTS_MESSAGE
            raise ex
          end

          logger.info "Service metadata already exists for csp org #{csp_organization_id}"
        end
      end

      private

      def get_slc_api(csp_organization_id)
        access_token = self.auth_client.fetch_access_token_using_oauth(csp_organization_id)
        return self.api_client.slc(access_token)
      end

      def create_grant_service_access_request(csp_organization_id, service_definition_id)
        attributes = {}
        attributes[:'org_id'] = csp_organization_id
        attributes[:'service_definition_id'] = service_definition_id
        attributes[:'is_tos_pre_signed'] = true
        service_access_request = ::CspSlc::GrantServiceAccessRequest::new(attributes)

        return service_access_request
      end
    end
  end
end