require 'import'
require 'time'
require 'json'
require 'tenant/models/csp_onboarding_stage_dto'
require 'tenant/errors/tenant_service_error'
require 'cht-lib-fabric/helpers/wavefront'

module Tenant
  module Stores
    class CspOnboardingStatusRepository
      include Import['logger',
                     'settings',
                     'dynamodb.client',
                     :wavefront]
      include ::Platform::Helpers::Wavefront

      WAVEFRONT_DYNAMO_DB_TIME_TAKEN_METRIC = 'com.cloudhealth.svc.tenant.csp-onboarding.api.dynamodb.time'
      WAVEFRONT_DYNAMO_DB_API_HIT_COUNT_METRIC = 'com.cloudhealth.svc.tenant.csp-onboarding.api.dynamodb.count'
      CSP_ONBOARDING_STATUS_TABLE_NAME = "cht-csp-onboarding-status"
      DEFAULT_RETRY_COUNT = 3
      DEFAULT_WAIT_TIME_IN_SEC = 5
      DEFAULT_REVISION = "latest"
      CURRENT_STATUS_GSI = "current-status-gsi"
      STATUS_TO_ID_MAP = {:new => 1, :csp_org_created => 2, :csp_org_persisted => 3,
                          :ch_service_associated => 4, :csp_metadata_updated => 5,
                          :admin_invitation_sent => 6, :completed => 7}

      def update_onboarding_status(tenant_id, tenant_name, csp_organization_id, csp_onboarding_stages_dto, updated_status)
        if updated_status.nil?
          raise Tenant::Errors::TenantServiceError.new(tenant_id, "Onboarding status is not present to update in dynamo db")
        end

        logger.info "Going to update status #{updated_status} for tenant id #{tenant_id}"

        last_updated_at = Time.now.utc.strftime("%Y-%m-%dT%H:%M:%S.%L")

        retry_count = 1
        begin
          if updated_status == :new
            add_status(tenant_id, tenant_name, csp_onboarding_stages_dto, updated_status, last_updated_at)
          else
            update_status(tenant_id, csp_organization_id, csp_onboarding_stages_dto, updated_status, last_updated_at)
          end
        rescue ::Tenant::Errors::TenantServiceError
          raise
        rescue => e
          if retry_count <= DEFAULT_RETRY_COUNT
            logger.error "Error occurred while updating csp onboarding status #{updated_status} for
                          tenant #{tenant_id}. Retrying.. Error: #{e.message}"
            sleep DEFAULT_WAIT_TIME_IN_SEC

            retry_count = retry_count + 1
            retry
          end

          logger.error "Unable to update csp onboarding status #{updated_status} for tenant id #{tenant_id}"
          raise
        end

        logger.info "Updated status #{updated_status} for tenant id #{tenant_id}"
      end

      def add_status(tenant_id, tenant_name, csp_onboarding_stages_dto, updated_status, last_updated_at)
        csp_onboarding_stages_dto_json = add_current_status(csp_onboarding_stages_dto, updated_status, last_updated_at)
        item = {
            tenantId: tenant_id,
            revision: DEFAULT_REVISION,
            tenantName: tenant_name,
            currentStatusId: STATUS_TO_ID_MAP[updated_status],
            currentStatus: updated_status,
            lastUpdatedAt: last_updated_at,
            statusHistory: csp_onboarding_stages_dto_json
        }

        params = {
            table_name: CSP_ONBOARDING_STATUS_TABLE_NAME,
            item: item
        }

        wf_time(WAVEFRONT_DYNAMO_DB_TIME_TAKEN_METRIC, tags: {:operation => 'put_item'}) do
          client.put_item(params)
        end

        wavefront.counter(WAVEFRONT_DYNAMO_DB_API_HIT_COUNT_METRIC, 1, operation: 'put_item')
      end

      def update_status(tenant_id, csp_organization_id, csp_onboarding_stages_dto, updated_status, last_updated_at)
        csp_onboarding_stages_dto_json = add_current_status(csp_onboarding_stages_dto, updated_status, last_updated_at)
        key = {
            tenantId: tenant_id,
            revision: DEFAULT_REVISION
        }
        update_expression = get_update_expression(updated_status)
        expression_attribute_values = get_update_expression_values(csp_organization_id, csp_onboarding_stages_dto_json, updated_status, last_updated_at)

        params = {
            table_name: CSP_ONBOARDING_STATUS_TABLE_NAME,
            key: key,
            update_expression: update_expression,
            expression_attribute_values: expression_attribute_values,
            return_values: nil
        }

        wf_time(WAVEFRONT_DYNAMO_DB_TIME_TAKEN_METRIC, tags: {:operation => 'update_item'}) do
          client.update_item(params)
        end

        wavefront.counter(WAVEFRONT_DYNAMO_DB_API_HIT_COUNT_METRIC, 1, operation: 'update_item')
      end

      def fetch_onboarding_status_for_all_tenants(exclusive_start_key = nil, onboarding_status)
        logger.info "Going to fetch csp onboarding status for all tenants with status id: #{onboarding_status}"
        params = {
            table_name: CSP_ONBOARDING_STATUS_TABLE_NAME,
            index_name: CURRENT_STATUS_GSI,
            exclusive_start_key: exclusive_start_key,
            select: "ALL_PROJECTED_ATTRIBUTES",
            key_condition_expression: 'revision = :revision and currentStatusId = :status_id',
            expression_attribute_values: {
                ':revision' => DEFAULT_REVISION,
                ':status_id' => onboarding_status
            }
        }

        begin
          response = nil
          wf_time(WAVEFRONT_DYNAMO_DB_TIME_TAKEN_METRIC, tags: {:operation => 'query'}) do
            response = client.query(params)
          end

          wavefront.counter(WAVEFRONT_DYNAMO_DB_API_HIT_COUNT_METRIC, 1, operation: 'query')

          logger.info "Fetched csp onboarding status for all tenants with status id: #{onboarding_status}"
          return response
        rescue
          logger.error "Unable to fetch csp onboarding status for all tenants"
          raise
        end
      end

      def fetch_onboarding_status_for_tenant(tenant_id)
        logger.info "Going to fetch csp onboarding status for tenant #{tenant_id}"
        params = {
            table_name: CSP_ONBOARDING_STATUS_TABLE_NAME,
            key: {
                tenantId: tenant_id,
                revision: DEFAULT_REVISION
            }
        }

        begin
          response = nil
          wf_time(WAVEFRONT_DYNAMO_DB_TIME_TAKEN_METRIC, tags: {:operation => 'get_item'}) do
            response = client.get_item(params)
          end

          wavefront.counter(WAVEFRONT_DYNAMO_DB_API_HIT_COUNT_METRIC, 1, operation: 'get_item')

          logger.info "Fetched csp onboarding status for tenant #{tenant_id}"
          return response
        rescue
          logger.error "Unable to fetch csp onboarding status for tenant #{tenant_id}"
          raise
        end
      end

      #
      # Resets a workflow by deleting it's entry in the status table.  This should only
      # be used for workflows in a new state.
      def reset_workflow(tenant_id, current_status_id = 1, current_status = "new")
        client.delete_item({
          table_name: CSP_ONBOARDING_STATUS_TABLE_NAME,
          key: {
            "tenantId" => tenant_id.to_i,
            "revision" => DEFAULT_REVISION
          }
        })
      end

      private

      def add_current_status(csp_onboarding_stages_dto, updated_status, last_updated_at)
        csp_onboarding_stage_dto = ::Tenant::Models::CspOnboardingStageDto.new(STATUS_TO_ID_MAP[updated_status], updated_status, last_updated_at)
        csp_onboarding_stages_dto.statuses.push(csp_onboarding_stage_dto)
        csp_onboarding_stages_dto_json = JSON.generate(csp_onboarding_stages_dto.to_hash)
        return csp_onboarding_stages_dto_json
      end

      def get_update_expression(updated_status)
        if updated_status == :csp_org_created
          return "set cspOrganizationId = :csp_organization_id, currentStatusId = :status_id, currentStatus = :current_status, lastUpdatedAt = :last_updated_at, statusHistory = :status_history"
        else
          return "set currentStatusId = :status_id, currentStatus = :current_status, lastUpdatedAt = :last_updated_at, statusHistory = :status_history"
        end
      end

      def get_update_expression_values(csp_organization_id, csp_onboarding_stages_dto_json, updated_status, last_updated_at)
        expression_attribute_values = {
            ":status_id" => STATUS_TO_ID_MAP[updated_status],
            ":current_status" => updated_status,
            ":last_updated_at" => last_updated_at,
            ":status_history" => csp_onboarding_stages_dto_json
        }

        if updated_status == :csp_org_created
          expression_attribute_values[':csp_organization_id'] = csp_organization_id
        end

        return expression_attribute_values
      end
    end
  end
end