# frozen_string_literal: true

require 'import'

require 'tenant/models'
require 'tenant/helpers/database'

module Tenant
  module Stores
    module DataStore
      # Keep constants in sync with cht_shared/lib/cht_shared/storage/data_store.rb
      # Maps to db values for data store names
      IRD = 'ird'
      MAIN = 'main'
      ES = 'es'
      AZURE_IRD = 'azure_ird'
      ES_METRICS = 'es_metrics'
      ES_METRICS_INDEX = 'es_metrics_index'

      SHARD_NAME_SET = ::Set[IRD, MAIN, ES,
                             AZURE_IRD, ES_METRICS, ES_METRICS_INDEX]
    end

    class ShardsDirectory
      include Import['cache.redis', 'alert']
      include ::Tenant::Helpers::Database

      class InvalidDataStore < ::RuntimeError; end
      class AssignmentNotFoundError < ::RuntimeError; end

      def by_customer_id_and_main(customer_id:)
        by_customer_id_and_store_name(customer_id: customer_id,
                                      data_store_name: DataStore::MAIN)
      end

      def by_customer_id_and_store_name(customer_id:,
                                        data_store_name:)
        raise InvalidDataStore unless DataStore::SHARD_NAME_SET.include?(data_store_name)

        begin
          cache_key = shard_cache_key(customer_id, data_store_name)
          shard_id = redis.get(cache_key)
          unless shard_id
            shard_id = get_shard_assignment_from_db(customer_id, data_store_name)
          end
        rescue ::Redis::TimeoutError, ::Redis::CommandError,
               ::Redis::CannotConnectError, ::Errno::ETIMEDOUT,
               ::Errno::ECONNREFUSED
          # If there are any redis connection issues just try to get it from sql without caching
          shard_id = get_shard_assignment_from_db(customer_id, data_store_name,
                                                  cache_result: false)
        end

        # For whatever reason we still can't find the shard assignment
        unless shard_id
          alert.error "Shard assignment not found",
                      customer_id: customer_id,
                      data_store_name: data_store_name
          raise AssignmentNotFoundError
        end

        to_shard_assignment(
          customer_id: customer_id,
          data_store_name: data_store_name,
          active_shard_id: shard_id
        )
      end

      private

      def get_shard_assignment_from_db(customer_id, data_store_name, cache_result: true)
        active_shard_id = nil
        with_db do |conn|
          results = conn.execute(<<~SQL
            SELECT shard_assignments.customer_id,
                   shard_assignments.shard_id,
                   data_stores.name
            FROM shard_assignments
            INNER JOIN data_stores ON shard_assignments.data_store_id = data_stores.id
            WHERE shard_assignments.customer_id = #{customer_id} AND
                  data_stores.name = '#{data_store_name}';
          SQL
          ).to_a

          active_shard_id = results.first['shard_id']
        end

        if cache_result
          cache_key = shard_cache_key(customer_id, data_store_name)
          redis.set(cache_key, active_shard_id, 1200) # 20 min ttl
        end

        active_shard_id
      end

      def shard_cache_key(customer_id, data_store_name)
        "#{customer_id}:#{data_store_name}"
      end

      def to_shard_assignment(customer_id:,
                              data_store_name:,
                              active_shard_id:)
        ::Tenant::Models::ShardAssignment.new(
          dataStoreName: data_store_name,
          activeShardId: active_shard_id.to_i
        )
      end
    end

  end
end
