require 'import'
require 'tenant/helpers/database'
require 'tenant/ar_models'
require 'tenant/errors/tenant_service_error'

module Tenant
  module Stores
    class TenantRepository
      include ::Tenant::Helpers::Database
      include Import['logger']

      DEFAULT_RETRY_COUNT = 3
      DEFAULT_WAIT_TIME_IN_SEC = 5
      CUSTOMER_ADMIN_ROLE_NAME = "customer_admin"

      def save_csp_organization_id(tenant_id, csp_organization_id)
        retry_count = 1
        begin
          logger.info "Persisting csp organization id #{csp_organization_id} for tenant #{tenant_id}"

          with_db do
            customer = ::Tenant::ArModels::Customer.where(
                id: tenant_id
            ).first

            if customer.nil?
              logger.error "No tenant exists with tenant id #{tenant_id}"
              raise ::Tenant::Errors::TenantServiceError.new(tenant_id, "Tenant does not exist")
            end

            # Customers onboarded through CSP directly and/ Customer Management should already have 
            # CSP organization ID, we do not want to overwrite it. Customers onboarded through the 
            # tenant service should be nil but not change after the fact. This should in theory 
            # never run but in the even a workflow was in progress we don't want it to complete here.
            raise ::Tenant::Errors::TenantServiceError.new(tenant_id, "Tenant already has a CSP organization id") unless customer.csp_organization_id.nil?

            customer.csp_organization_id = csp_organization_id
            customer.save!
          end
        rescue ::Tenant::Errors::TenantServiceError
          raise
        rescue => e
          if retry_count <= DEFAULT_RETRY_COUNT
            logger.error "Error occurred while persisting csp organization id #{csp_organization_id} for
                          tenant #{tenant_id}. Retrying.. Error: #{e.message}"
            sleep DEFAULT_WAIT_TIME_IN_SEC

            retry_count = retry_count + 1
            retry
          end

          logger.error "Unable to persist csp organization id #{csp_organization_id} for tenant #{tenant_id}"
          raise
        end

        logger.info "Persisted csp organization id #{csp_organization_id} for tenant #{tenant_id}"
      end

      def fetch_tenant_details(tenant_id)
        with_db do
          customer = ::Tenant::ArModels::Customer
                         .where(id: tenant_id).where(is_deleted: 0)
                         .select(:id, :name, :address1, :address2, :city, :zipcode, :is_active, :is_trial, :is_suspended, :trial_expiration,
                                 :max_groupsets_for_viz, :classification, :distributor_id, :reseller_id, :service_provider_id,
                                 :is_msp, :pgb_on, :reseller_discount_percent, :cloud, :cloud_provider, :is_cloud_provider, :default_cloud,
                                 :time_zone_utc_offset, :billing_contact, :tags, :benchmark_price_enabled_at, :csp_organization_id)
                         .first

          if customer.nil?
            logger.error "No tenant exists with tenant id #{tenant_id}"
            raise ::Tenant::Errors::TenantServiceError.new(tenant_id, "Tenant does not exist")
          end

          return customer.attributes
        end
      end

      def fetch_admins_for_tenant(tenant_ids)
        logger.info "Fetching tenant admins for tenant ids #{tenant_ids}"

        tenant_id_to_admin_map = {}
        with_db do
          users = ::Tenant::ArModels::User.joins(:customer, :roles)
                      .where(customers: {id: tenant_ids})
                      .where(users: {is_deleted: 0})
                      .where(roles: {is_builtin: 1})
                      .where(roles: {name: CUSTOMER_ADMIN_ROLE_NAME})

          if ! users.nil?
            users.each do |user|
              admins = tenant_id_to_admin_map[user.customer_id]
              if admins.nil?
                admins = []
                tenant_id_to_admin_map[user.customer_id] = admins
              end
              admins.push(user.email)
            end
          end
        end

        return tenant_id_to_admin_map
      end

      def fetch_tenants_without_csp_organization
        with_db do
          tenants = ::Tenant::ArModels::Customer
                       .where(csp_organization_id: nil)
                       .where(is_active: 1)
                       .where(is_deleted: 0)
                       .select(:id, :name, :address1, :address2, :city, :zipcode, :is_active, :is_trial, :is_suspended, :trial_expiration,
                               :max_groupsets_for_viz, :classification, :distributor_id, :reseller_id, :service_provider_id,
                               :is_msp, :pgb_on, :reseller_discount_percent, :cloud, :cloud_provider, :is_cloud_provider, :default_cloud,
                               :time_zone_utc_offset, :billing_contact, :tags, :benchmark_price_enabled_at, :csp_organization_id)

          logger.info "Number of tenants without an associated csp organization = #{tenants.length}"
          return tenants
        end
      end
    end
  end
end