# frozen_string_literal: true

require 'import'
require 'cht-lib-fabric/crns'
require 'tenant/tasks/legacy_cache'
require 'multidb'

module Tenant
  module Stores
    class TenantsDirectory
      include Import['logger', 'cache.redis']
      include ::Tenant::Helpers::Database

      def from_customer_id(id)
        result = to_tenant(redis.deserialize(id))
        # Do one fallback check
        if result.nil?
          ::Tenant::Tasks::LegacyCache.new.do_customer_with_tenant_lookup(id)
          result = to_tenant(redis.deserialize(id))
        end
        logger.warn "Tenant [#{id}] was not found in cache" if result.nil?
        result
      end

      def to_tenant(raw_hash)
        return nil unless raw_hash

        tenant = ::Tenant::Models::Tenant.new(
          id: ::Platform::CRNs::Tenant.new(customer: raw_hash['id']),
          name: raw_hash['name'],
          isMsp: !raw_hash['is_msp'].nil? && !raw_hash['is_msp'].zero?,
          serviceProviderId: if raw_hash['service_provider_id']
                               ::Platform::CRNs::Tenant.new(customer: raw_hash['service_provider_id'])
                             end,
          sessionLength: raw_hash['session_length'],
          customerTenantSupportLink: true
        )
        # Lookup partner setting (if any) and set that value
        # which could still be true or false. If the partner_settings is nil then we'll let the
        # default of true apply
        partner_settings = do_partner_tenant_settings(tenant)
        unless partner_settings.nil?
          logger.info("Partner Settings #{ partner_settings.to_json }")
          # Update struct here or set it before hand
          tenant.attributes.merge!(customerTenantSupportLink: partner_settings.customer_tenant_support_link)
        end
        logger.info("Tenant = #{tenant.to_json}")
        # return the modified tenant
        tenant

      end

      # 
      #
      # NOTE: This is looking up the partner settings of the partner, not the partner customer
      #
      # Lazily lookup partner settings when node query is run.
      # @param tenant - The tenant we want to check partner settings for
      # @param partner_settings - The tenants partner settings or nil if they do not have any
      #
      def do_partner_tenant_settings(tenant)
        logger.info("Do Partner Settings for #{tenant.to_json} #{tenant.class.name}")
        # Only do this stuff for tenants that are partner customers
        unless tenant.serviceProviderId.nil?

          # Use the customer ID of the partner, not the partner customer
          # This is because for the support link functionality we use the partners
          # setting. If we need to support looking at other partner settings we'll 
          # need to generalize this a bit more.
          customer_id  = tenant.serviceProviderId.customer

          # Lookup customer shard here
          customer_shard = Multidb.use("master") do
            # TODO: Should lookup by name via DataStore
            ::Tenant::ArModels::ShardAssignment.where(customer_id: customer_id, data_store_id: 2).first!
          end

          logger.info("Customer shard found for #{customer_id} = #{customer_shard.to_json}")

          partner_settings = Multidb.use("db#{customer_shard.shard_id}") do
            # We'll let this come back null if the partner has no settings
            ::Tenant::ArModels::PartnerSettings.where(customer_id: customer_id).first
          end
          # return partner settings here
          return partner_settings

        end
        # not a partner customer so no settings
        nil
      end

    end
  end
end
