require 'import'
require 'ostruct'
require 'json'
require 'tenant/producers/csp_backfill_message_producer'

module Tenant
   module Tasks
      class CreateMissingCspWorkflow
         include Import['logger', 'settings', 'stores.csp_onboarding_status_repo', 'handlers.rollbar_handler', 'stores.tenant_repository']

         ACTION_TYPE = "csp-back-fill"
         STATUS_TO_ID_MAP = {:new => 1, :csp_org_created => 2}
         TENANT_COUNT_TO_SLEEP = 30
         SLEEP_DURATION = 5

         def execute
            logger.info "Creating CSP onboarding workflow for missing tenants"
            begin
               tenants_with_csp_workflows_existing = []

               STATUS_TO_ID_MAP.each do |k, v|
                  last_evaluated_key = nil

                  loop do
                     response = csp_onboarding_status_repo.fetch_onboarding_status_for_all_tenants(last_evaluated_key, v)
                     results = response.items
                     last_evaluated_key = response.last_evaluated_key

                     results.each do |item|
                        workflow_detail = JSON.parse(item.to_json, object_class: OpenStruct)
                        tenants_with_csp_workflows_existing.push(workflow_detail.tenantId.to_i)
                     end

                     break if last_evaluated_key.nil?
                  end
               end

               create_workflow_for_tenants_without_workflow(tenants_with_csp_workflows_existing)

               logger.info "Created CSP onboarding workflow for missing tenants"
            rescue => ex
               logger.error "Error occurred while creating csp onboarding workflows for missing tenants. Error: #{([ex.message]+ex.backtrace).join($/)}"
               rollbar_handler.handle(ex)
            end
         end

         def create_workflow_for_tenants_without_workflow(tenant_ids_with_workflow)
            tenants_without_csp_organization = tenant_repository.fetch_tenants_without_csp_organization
            @producer = ::Tenant::Producers::CspBackfillMessageProducer.new

            count = 1
            tenants_without_csp_organization.each do |tenant|
               tenant_id = tenant.id
               if tenant_ids_with_workflow.include? tenant_id
                  logger.info "CSP onboarding workflow already exists for tenant id #{tenant_id}"
                  next
               end

               begin
                  @producer.push(tenant_id, tenant.attributes, ACTION_TYPE)
                  logger.info "Created missing csp workflow for tenant id #{tenant_id}"
                  if count % TENANT_COUNT_TO_SLEEP == 0
                     sleep SLEEP_DURATION
                  end
                  count = count+1
               rescue => ex
                  logger.error "Error occurred while creating missing csp onboarding workflow for tenant id #{tenant_id}. Error: #{([ex.message]+ex.backtrace).join($/)}"
               end
            end
         end
      end
   end
end
