require 'import'
require 'tenant/models/csp_onboarding_stages_dto'
require 'tenant/clients/csp_org_client'
require 'tenant/clients/csp_slc_client'
require 'tenant/errors/tenant_service_error'
require 'cht-lib-fabric/helpers/wavefront'

module Tenant
  module Tasks
    class CspOnboardingWorkflow
      include Import['logger',
                     'settings',
                     'stores.tenant_repository',
                     'stores.csp_onboarding_status_repo',
                     'handlers.rollbar_handler',
                     :wavefront]
      include ::Platform::Helpers::Wavefront

      attr_accessor :org_client, :slc_client

      def csp_onboarding(tenant_id, tenant_name = nil, csp_organization_id = nil , cht_onboarding_stages_dto = nil,
                         onboarding_status = nil, status_update_required = true, action_type = nil, admin_invitation_to_be_sent = true, admins = [])

        begin
          if onboarding_status.nil?
            response = csp_onboarding_status_repo.fetch_onboarding_status_for_tenant(tenant_id)
            if !response.item.nil?
              logger.info "CSP onboarding workflow already exists for tenant id #{tenant_id}"
              return
            end

            if "create".eql?(action_type)
              wavefront.counter('com.cloudhealth.svc.tenant.csp-onboarding.message.new.count')
            else
              wavefront.counter('com.cloudhealth.svc.tenant.csp-onboarding.message.backfill.count')
            end

            logger.info "CSP onboarding workflow does not exist for tenant id #{tenant_id}. Proceeding.."
            onboarding_status = :new
            cht_onboarding_stages_dto = ::Tenant::Models::CspOnboardingStagesDto.new
          end

          if status_update_required
            csp_onboarding_status_repo.update_onboarding_status(tenant_id, tenant_name, csp_organization_id, cht_onboarding_stages_dto, onboarding_status)
          end

          self.org_client = ::Tenant::Clients::CspOrgClient::new
          self.slc_client = ::Tenant::Clients::CspSlcClient::new

          wf_time('com.cloudhealth.svc.tenant.csp-onboarding.time', tags: {:stage => onboarding_status.to_s}) do
            case onboarding_status

            when :new
              logger.info "Starting csp onboarding for tenant ID #{tenant_id}."
              csp_organization_id = org_client.create_csp_organization(tenant_id, tenant_name)
              csp_onboarding(tenant_id, tenant_name, csp_organization_id, cht_onboarding_stages_dto, :csp_org_created)

            when :csp_org_created
              tenant_repository.save_csp_organization_id(tenant_id, csp_organization_id)
              csp_onboarding(tenant_id, tenant_name, csp_organization_id, cht_onboarding_stages_dto, :csp_org_persisted)

            when :csp_org_persisted
              slc_client.grant_service_access(csp_organization_id, settings.ch_service_definition_id)
              csp_onboarding(tenant_id, tenant_name, csp_organization_id, cht_onboarding_stages_dto, :ch_service_associated)

            when :ch_service_associated
              slc_client.add_service_organization_metadata(tenant_id, csp_organization_id, settings.ch_service_definition_id)
              csp_onboarding(tenant_id, tenant_name, csp_organization_id, cht_onboarding_stages_dto, :csp_metadata_updated)

            when :csp_metadata_updated
              if !admin_invitation_to_be_sent
                return
              end

              if admins.length() == 0
                tenant_id_to_admin_map = tenant_repository.fetch_admins_for_tenant([tenant_id])
                admins = tenant_id_to_admin_map[tenant_id]
              end

              if admins.nil? or admins.length() == 0
                logger.info "There are no admins to invite to csp organization #{csp_organization_id} for tenant #{tenant_id}"
                return
              end

              org_client.invite_users_to_csp_organization(csp_organization_id, admins)
              csp_onboarding(tenant_id, tenant_name, csp_organization_id, cht_onboarding_stages_dto, :admin_invitation_sent)

            when :admin_invitation_sent
              # Find if this csp organization has org owners other than csp.local account
              users_dto = org_client.get_users_in_csp_organization(csp_organization_id)
              if users_dto.nil?
                logger.info "CH admin account is already removed from csp organization id #{csp_organization_id}"
                csp_onboarding(tenant_id, tenant_name, csp_organization_id, cht_onboarding_stages_dto, :completed)
              else
                owners_count = org_client.find_number_of_org_owners(users_dto, tenant_id, csp_organization_id)
                logger.debug "Number of org owners for csp organization id #{csp_organization_id} = #{owners_count}"

                if(owners_count <= 1)
                  logger.info "There is no org owner from tenant organization yet for csp organization id #{csp_organization_id}"
                else
                  org_client.remove_users_from_csp_organization(tenant_id, csp_organization_id, [settings.csp_account_username])
                  csp_onboarding(tenant_id, tenant_name, csp_organization_id, cht_onboarding_stages_dto, :completed)
                end
              end

            when :completed
              logger.info "Completed csp onboarding for tenant ID #{tenant_id}"

            end
          rescue => ex
            logger.error "Error occurred while onboarding tenant id #{tenant_id} to csp. Error: #{([ex.message]+ex.backtrace).join($/)}"
            rollbar_handler.handle(ex, tenant_id, csp_organization_id)
          end

          end
      end

    end
  end
end