# frozen_string_literal: true

require 'import'
require 'sidekiq'
require 'erb'

require 'tenant/helpers/database'

module Tenant
  module Tasks
    class LegacyCache
      include Import['logger', 'cache.redis']
      include ::Sidekiq::Worker
      include ::Tenant::Helpers::Database

      SHARD_ASSIGNMENT_TTL = 1200 # 20 minutes

      def perform(_options)
        with_db do |conn|
          logger.info "Getting all tenant info"
          results = conn.execute(
            '
              SELECT
                customers.id,
                customers.name,
                customers.is_msp,
                customers.service_provider_id,
                customers.session_length,
                partner_settings.customer_tenant_support_link
              FROM
                customers
              LEFT JOIN
                partner_settings ON customers.service_provider_id=partner_settings.customer_id
              WHERE
                customers.is_deleted is false
            '
          ).to_a
          logger.info "#{results.length} tenants received, pushing into Redis"

          before_keys = redis.keys
          results.each do |tenant|
            logger.debug "Pushing #{tenant['name']} #{tenant['id']} into redis"
            do_customer(tenant)
          end

          keys_to_delete = before_keys - results.map { |t| t['id'] }.compact.map(&:to_s)
          logger.info "#{keys_to_delete.length} tenants were in Redis but no longer in DB and will be deleted"
          keys_to_delete.each do |key|
            logger.debug "Deleting tenant #{key}"
            redis.del(key)
          end
        end
      end

      #
      # Processes a single tenant setting their supported feature flags
      # and shard assignment
      #
      def do_customer(tenant)
        logger.debug "Tenant::LegacyCache::doCustomer #{tenant}"
        logger.debug "Pushing settings for tenant #{tenant['id']} into redis"
        redis.serialize(tenant['id'], tenant)
      end

      #
      # Process a single tenant and lookup their info first
      #
      def do_customer_with_tenant_lookup(customer_id)
        logger.info("do_customer_with_tenant_lookup #{customer_id}")
        if customer_id
          with_db do |conn|
            results = conn.execute(
              "
                SELECT
                  customers.id,
                  customers.name,
                  customers.is_msp,
                  customers.service_provider_id,
                  customers.session_length
                FROM
                  customers
                WHERE
                  customers.is_deleted is false and customers.id = #{customer_id}
              "
            ).to_a
            logger.info "do_customer_with_tenant_lookup results = [#{results}]"
            if (results.nil? || results.empty?)
              logger.error("Unable to find customer #{customer_id} in database during onboarding")
            end
            do_customer(results[0]) unless results.nil? || results.empty?
          end
        end
      end

      # Cache all shard ids into redis by customer and data store name
      def cache_shard_assignments
        logger.info "Starting cache shard assignments task..."

        with_db do |conn|
          results = conn.execute(<<~SQL
            SELECT shard_assignments.customer_id,
                   shard_assignments.shard_id,
                   data_stores.name
            FROM shard_assignments
            INNER JOIN data_stores ON shard_assignments.data_store_id = data_stores.id;
          SQL
          ).to_a

          results.each do |shard_data|
            key = "#{shard_data['customer_id']}:#{shard_data['name']}"
            value = shard_data['shard_id']

            logger.debug "Setting #{key} with #{value}"

            redis.set(key, value, SHARD_ASSIGNMENT_TTL)
          end
        end
      end

    end
  end
end
