require 'import'
require 'ostruct'
require 'json'
require 'tenant/clients/csp_org_client'
require 'tenant/clients/csp_user_client'
require 'tenant/tasks/csp_onboarding_workflow'
require 'tenant/models/csp_onboarding_stage_dto'
require 'tenant/models/csp_onboarding_stages_dto'

module Tenant
  module Tasks
    class ResumeCspWorkflow
      include Import['logger', 'settings', 'stores.csp_onboarding_status_repo', 'handlers.rollbar_handler', 'stores.tenant_repository']

      NEW_STATUS_ID = 1
      CSP_METADATA_UPDATED_STATUS_ID = 5
      STATUS_TO_ID_MAP = {:csp_org_created => 2, :csp_org_persisted => 3, :ch_service_associated => 4, :admin_invitation_sent => 6}

      attr_accessor :workflow, :org_client, :user_client

      def resume_workflow_for_all_tenants
        logger.info "Resuming onboarding workflow for all tenants"
        @workflow = ::Tenant::Tasks::CspOnboardingWorkflow.new

        begin
          csp_organization_ids_existing = []

          STATUS_TO_ID_MAP.each do |k, v|
            count = 0
            last_evaluated_key = nil

            loop do
              response = csp_onboarding_status_repo.fetch_onboarding_status_for_all_tenants(last_evaluated_key, v)
              results = response.items
              last_evaluated_key = response.last_evaluated_key
              count = count + results.length()

              results.each do |item|
                workflow_detail = JSON.parse(item.to_json, object_class: OpenStruct)

                csp_organization_ids_existing.push(workflow_detail.cspOrganizationId)

                # Temporarily do not  resume workflow for status = 5/6 since a lot of tenants are waiting in that
                # state currently without any action that can be taken
                if workflow_detail.currentStatusId.to_i != 6
                  resume_workflow_for_specific_tenant(workflow_detail)
                end
              end

              break if last_evaluated_key.nil?
            end

            logger.debug "Number of tenants with onboarding status: '#{k}' = #{count}"
          end

          # Fetch onboarding status for tenants separately for which admin invitations need to be sent.
          # Then just fetch admins for all tenants using a single query.
          # Then resume workflow for each of these tenants.
          send_admin_invitations(csp_organization_ids_existing)

          resume_workflow_for_new_tenants(csp_organization_ids_existing)

          logger.info "Resumed onboarding workflow for all tenants"
        rescue => ex
          logger.error "Error occurred while resuming onboarding workflow for all tenants. Error: #{([ex.message]+ex.backtrace).join($/)}"
          rollbar_handler.handle(ex)
        end
      end

      def send_admin_invitations(csp_organization_ids_existing)
        count = 0
        last_evaluated_key = nil

        loop do
          tenant_id_to_workflow_map = {}
          response = csp_onboarding_status_repo.fetch_onboarding_status_for_all_tenants(last_evaluated_key, CSP_METADATA_UPDATED_STATUS_ID)
          last_evaluated_key = response.last_evaluated_key
          results = response.items
          count = count + results.length()

          results.each do |item|
            workflow_detail = JSON.parse(item.to_json, object_class: OpenStruct)
            tenant_id_to_workflow_map[workflow_detail.tenantId.to_i] = workflow_detail
            csp_organization_ids_existing.push(workflow_detail.cspOrganizationId)
          end

          if tenant_id_to_workflow_map.keys.length() == 0
            logger.debug "There are no tenants for which admin invitations need to be sent."
            return
          end

          # Temporarily do not send invitations
          #tenant_id_to_admin_map = tenant_repository.fetch_admins_for_tenant(tenant_id_to_workflow_map.keys)
          #tenant_id_to_workflow_map.each do |k, v|
          #  admins = tenant_id_to_admin_map[k]
          #  if !admins.nil? and admins.length() > 0
          #    resume_workflow_for_specific_tenant(v, nil, nil, false, true, admins)
          #  end
          #end

          break if last_evaluated_key.nil?
        end

        logger.debug "Number of tenants waiting for admin invitations to be sent = #{count}"
      end

      def resume_workflow_for_new_tenants(csp_organization_ids_existing)
        # If there are tenants with new status, then only:
        #   1. Get all csp organizations list
        #   2. Remove csp organizations which are already present in dynamo db.
        #   3. For each of remaining csp organization, fetch org details and extract tenant id
        #   4. If for a tenant, csp organization id is found, then don't create a new csp organization and just proceed from next step.

        last_evaluated_key = nil
        new_tenants_exist = false
        tenant_id_to_csp_organization_id_map = {}

        loop do
          response = csp_onboarding_status_repo.fetch_onboarding_status_for_all_tenants(last_evaluated_key, NEW_STATUS_ID)
          results = response.items
          last_evaluated_key = response.last_evaluated_key

          if !new_tenants_exist and results.length() > 0
            csp_organization_ids = fetch_csp_organizations
            csp_organization_ids -= csp_organization_ids_existing
            tenant_id_to_csp_organization_id_map = map_tenant_id_to_csp_organization_id(csp_organization_ids)
            new_tenants_exist = true
          end

          results.each do |item|
            workflow_detail = JSON.parse(item.to_json, object_class: OpenStruct)
            csp_organization_id = tenant_id_to_csp_organization_id_map[workflow_detail.tenantId.to_i]
            if csp_organization_id.nil?
              resume_workflow_for_specific_tenant(workflow_detail)
            else
              resume_workflow_for_specific_tenant(workflow_detail, csp_organization_id, :csp_org_created, true)
            end
          end

          break if last_evaluated_key.nil?
        end
      end

      def resume_workflow_for_specific_tenant(workflow_detail, overridden_csp_organization_id = nil, overridden_status = nil, status_update_required = false, admin_invitation_to_be_sent = false, admins = [])
        tenant_id = workflow_detail.tenantId.to_i
        tenant_name = workflow_detail.tenantName

        csp_organization_id = workflow_detail.cspOrganizationId
        if !overridden_csp_organization_id.nil?
          csp_organization_id = overridden_csp_organization_id
        end

        current_onboarding_status = workflow_detail.currentStatus
        if !overridden_status.nil?
          current_onboarding_status = overridden_status
        end

        csp_onboarding_stages_dto = extract_csp_onboarding_stages(workflow_detail)

        logger.info "Resuming onboarding workflow for tenant id #{tenant_id}"
        @workflow.csp_onboarding(tenant_id, tenant_name, csp_organization_id, csp_onboarding_stages_dto, current_onboarding_status.to_sym, status_update_required, nil, admin_invitation_to_be_sent, admins)
      end

      private

      def fetch_csp_organizations
        @user_client = ::Tenant::Clients::CspUserClient::new
        csp_organization_ids = []
        response = @user_client.get_csp_organizations_for_a_user(settings.csp_account_username)
        response.ref_links.each do |ref_link|
          csp_organization_id = ref_link.split('/').last()
          csp_organization_ids.push(csp_organization_id)
        end

        return csp_organization_ids
      end

      def map_tenant_id_to_csp_organization_id(csp_organization_ids)
        @org_client = ::Tenant::Clients::CspOrgClient::new
        map = {}
        csp_organization_ids.each do |csp_organization_id|
          csp_org_detail = @org_client.get_csp_organization_details(csp_organization_id)
          tenant_id = @org_client.extract_tenant_id_from_org_display_name(csp_org_detail)
          if !tenant_id.nil?
            tenant_id = tenant_id.to_i
          end
          map[tenant_id] = csp_organization_id
        end

        return map
      end

      def extract_csp_onboarding_stages(workflow_detail)
        stages = []
        status_history = JSON.parse(workflow_detail.statusHistory, object_class: OpenStruct)

        status_history.statuses.each do |status|
          csp_onboarding_stage = ::Tenant::Models::CspOnboardingStageDto.new(status.id.to_i, status.status, status.createdOn)
          stages.push(csp_onboarding_stage)
        end

        return ::Tenant::Models::CspOnboardingStagesDto.new(stages)
      end
    end
  end
end